function [percentStrain,deltaLi,lengthNi,filename] = PercentStrain_3D

% Created by: Virginia Orozco, MS

%% Inputs and Filenmae
% Input rupture time (tNf) and number of tracked numbers (markers)
% Select xyz pts .cvs filename

% Enter time, i.e., Frame = 347 --> tNf = 3.47:
tNf = input('Enter the rupture time (tNf):');
% Enter the total number of markers tracked:
markers = input('Enter the number of tracked markers (markers):');
% Selecte filename with *_xyzpts.cvs:
filename = uigetfile('*_xyzpts.csv');

%% Import [X,Y,Z] trajectories (*.csv file [mm])
% Calculate the new nerve length (Lni [mm])
% Calculate the length between the markers, using the distance formula.
%     d = sqrt( (x2-x1)^2 + (y2-y1)^2 + (z2-z1)^2 )
% Full: defined as insertion marker to clamp marker
% Section #1: defined as insertion marker to marker 1
% Section #2: defined as marker 1 to marker 2
% Section #3: defined as marker 2 to marker 3
% Section #4: defined as marker 3 to marker 4
% and so on ...

num = readmatrix(filename);
num(any(isnan(num), 2), :) = [];

lengthNi = zeros(size(num,1),size(markers,2)); % Column 1 of the final matrix containing iterative distances - specifically between insertion and clamp
% Length between Insertion to Clamp:
lengthNi(:,1) = sqrt( (num(:,size(num,2)-2) - num(:,1)).^2 + ...      % X
                      (num(:,size(num,2)-1) - num(:,2)).^2 + ...      % Y
                      (num(:,size(num,2))   - num(:,3)).^2 );         % Z 
idx = 2; % Column number that contains iterative distances between the corresponding timepoints of any two markers (inclusive of insertion/clamp) 
% Length in-between markers:
for i = 4:3:size(num,2)-2
    lengthNi(:,idx) = sqrt( (num(:,i)   - num(:,i-3)).^2 + ...   % X
                            (num(:,i+1) - num(:,i-2)).^2 + ...   % Y
                            (num(:,i+2) - num(:,i-1)).^2);       % Z
    idx = idx + 1;
end

%% Calcualte change in length and percent strain
% Calculate the iterative change in nerve length (deltaLi [mm]).
% Then, using the iterative change in nerve length, determine the iterative
% percent strain by dividing deltaLi by the initial length (lengthNi(1,:)).

deltaLi = zeros(size(lengthNi));       % Empty array for iterative delta nerve lengths
deltaLi(1,:) = 0;                      % Initialize iterative delta nerve length = 0
percentStrain = zeros(size(lengthNi)); % Empty array for iterative percent strain
percentStrain(1,:) = 0;                % Initialize iterative percent strain = 0

for i = 1:size(lengthNi,2)
    for idx = 2:size(num,1)
        deltaLi(idx,i) = lengthNi(idx,i)-lengthNi(1,i);
        percentStrain(idx,i) = (deltaLi(idx,i)/(lengthNi(1,i)))*100; 
    end
end

%% Plot length vs time
time = 0:1/100:(tNf-0.01);
markers = 1:size(lengthNi,2);
FIG = figure(1);
parentDir = cd;
savePath = uigetdir('C:\Users\virgi\OneDrive\Desktop\2D vs 3D %strain - UDPATE',...
    'Select Folder To Save Strain Figures');
for i = 1:numel(markers)
    figure(FIG)
    plot(time,lengthNi(:,i),'k','LineWidth',1.5);
    title(sprintf('Length %d',markers(i)),'FontSize',16);
    xlabel('Time [s]','FontSize',16); 
    ylabel('Length [mm]','FontSize',16);
    cd(savePath)
    saveas(gca,sprintf('Length %d.tiff',markers(i)),'tiff')
    cd(parentDir)
end

%% Plot change in legnth vs time 
time = 0:1/100:(tNf-0.01);
markers = 1:size(percentStrain,2);
FIG = figure(1);
parentDir = cd;
savePath = uigetdir('C:\Users\virgi\OneDrive\Desktop\2D vs 3D %strain - UDPATE',...
    'Select Folder To Save Strain Figures');
for i = 1:numel(markers)
    figure(FIG)
    plot(time,deltaLi(:,i),'k','LineWidth',1.5);
    title(sprintf('\\Delta L %d',markers(i)),'FontSize',16);
    xlabel('Time [s]','FontSize',16); 
    ylabel('\Delta L [mm]','FontSize',16);
    cd(savePath)
    saveas(gca,sprintf('deltaL %d.tiff',markers(i)),'tiff')
    cd(parentDir)
end

%% Plot percent strain vs time
time = 0:1/100:(tNf-0.01);
markers = 1:size(percentStrain,2);
FIG = figure(1);
parentDir = cd;
savePath = uigetdir('C:\Users\virgi\OneDrive\Desktop\2D vs 3D %strain - UDPATE',...
    'Select Folder To Save Strain Figures');
for i = 1:numel(markers)
    figure(FIG)
    plot(time,percentStrain(:,i),'k','LineWidth',1.5);
    title(sprintf('Strain %d',markers(i)),'FontSize',16);
    xlabel('Time [s]','FontSize',16); 
    ylabel('% Strain','FontSize',16);
    cd(savePath)
    saveas(gca,sprintf('Strain %d.tiff',markers(i)),'tiff')
    cd(parentDir)
end

%% Export length, change in legnth, and percent strain to Excel Sheet
xlswrite(strcat(filename(1:19),'_3Dstrain','.xls'),[time' lengthNi],'Length');
xlswrite(strcat(filename(1:19),'_3Dstrain','.xls'),[time' deltaLi],'deltaL');
xlswrite(strcat(filename(1:19),'_3Dstrain','.xls'),[time' percentStrain],'PercentStrain');